@@ -184,7 +184,7 @@ class MISOSampler:
                     lag=2,
                     prior_params=None, 
                     start_cond=pysplicing.MISO_START_AUTO,
-                    stop_cond=pysplicing.MISO_STOP_CONVERGENT_MEAN,
+                    stop_cond=pysplicing.MISO_STOP_FIXEDNO,
                     verbose=True):
         """
         Fast version of MISO MCMC sampler.
@@ -291,7 +291,8 @@ class MISOSampler:
                                            prior_params, 
                                            long(self.overhang_len),
                                            long(num_chains),
-                                           start_cond, stop_cond)
+                                           start_cond,
+                                           stop_cond)
 
         # Psi samples
         psi_vectors = transpose(array(miso_results[0]))
