@@ -131,7 +131,7 @@ int splicing_matchIso_paired(const splicing_gff_t *gff, int gene,
     SPLICING_CHECK(splicing_vector_init(&vfragmentProb, 0));
     SPLICING_FINALLY(splicing_vector_destroy, &vfragmentProb);
     SPLICING_CHECK(splicing_normal_fragment(normalMean, normalVar, numDevs, 
-					    2*readLength, myfragmentProb,
+					    readLength, myfragmentProb,
 					    &fragmentStart));
     splicing_vector_scale(myfragmentProb, 
 			  1.0/splicing_vector_sum(myfragmentProb));
@@ -281,11 +281,13 @@ int splicing_solve_gene(const splicing_gff_t *gff, size_t gene,
 			const splicing_vector_int_t *position, 
 			const char **cigarstr,
 			splicing_matrix_t *match_matrix,
+			splicing_vector_t *nomatch,
 			splicing_matrix_t *assignment_matrix, 
-			splicing_vector_t *expression) {
-
+			splicing_vector_t *expression,
+			splicing_vector_t *residuals, int scale) {
+  
   splicing_matrix_t A;
-  splicing_vector_t match;
+  splicing_vector_t match, *mynomatch=nomatch, vnomatch;
   splicing_vector_long_t index;
   size_t no_classes;
   size_t no_reads=splicing_vector_int_size(position);
@@ -296,6 +298,12 @@ int splicing_solve_gene(const splicing_gff_t *gff, size_t gene,
   splicing_matrix_t *mymatch_matrix=match_matrix, vmatch_matrix;
   splicing_matrix_t *myass_matrix=assignment_matrix, vass_matrix;
 
+  if (!nomatch) {
+    mynomatch=&vnomatch;
+    SPLICING_CHECK(splicing_vector_init(mynomatch, 0));
+    SPLICING_FINALLY(splicing_vector_destroy, mynomatch);
+  }
+
   if (!match_matrix) {
     mymatch_matrix=&vmatch_matrix;
     SPLICING_CHECK(splicing_matrix_init(mymatch_matrix, 0, 0));
@@ -337,6 +345,20 @@ int splicing_solve_gene(const splicing_gff_t *gff, size_t gene,
   SPLICING_CHECK(splicing_matrix_transpose(&A));
   SPLICING_CHECK(splicing_vector_long_init(&index, 0));
   SPLICING_FINALLY(splicing_vector_long_destroy, &index);
+  splicing_vector_update(mynomatch, &match);
+    
+  SPLICING_CHECK(splicing_nnls(&A, &match, expression, &rnorm, &index, 
+			       &nsetp));
+
+  if (residuals) {
+    int i, n=splicing_vector_size(&match);
+    splicing_dgemv(/*transpose=*/ 1, /*alpha=*/ 1.0, myass_matrix, 
+		   expression, /*beta=*/ 0.0, &match);
+    SPLICING_CHECK(splicing_vector_resize(residuals, no_classes));
+    for (i=0; i<n; i++) {
+      VECTOR(*residuals)[i] = VECTOR(*mynomatch)[i] - VECTOR(match)[i];
+    }
+  }
 
   if (!assignment_matrix) {
     splicing_matrix_destroy(myass_matrix);
@@ -346,16 +368,21 @@ int splicing_solve_gene(const splicing_gff_t *gff, size_t gene,
     splicing_matrix_destroy(mymatch_matrix);
     SPLICING_FINALLY_CLEAN(1);
   }
-  
-  SPLICING_CHECK(splicing_nnls(&A, &match, expression, &rnorm, &index, 
-			       &nsetp));
-  
+
+  if (!nomatch) { 
+    splicing_vector_destroy(mynomatch);
+    SPLICING_FINALLY_CLEAN(1);
+  }
+
   splicing_vector_long_destroy(&index);
   splicing_matrix_destroy(&A);
   splicing_vector_destroy(&match);
   SPLICING_FINALLY_CLEAN(3);
 
-  splicing_vector_scale(expression, 1.0/splicing_vector_sum(expression));
+  if (scale) {
+    double fac=1.0/splicing_vector_sum(expression);
+    splicing_vector_scale(expression, fac);
+  }
 
   return 0;
 }
@@ -368,11 +395,13 @@ int splicing_solve_gene_paired(const splicing_gff_t *gff, size_t gene,
 			       int fragmentStart, double normalMean,
 			       double normalVar, double numDevs,
 			       splicing_matrix_t *match_matrix,
+			       splicing_vector_t *nomatch,
 			       splicing_matrix_t *assignment_matrix,
-			       splicing_vector_t *expression) {
+			       splicing_vector_t *expression, 
+			       splicing_vector_t *residuals, int scale) {
   
   splicing_matrix_t A;
-  splicing_vector_t match;
+  splicing_vector_t match, *mynomatch=nomatch, vnomatch;
   splicing_vector_long_t index;
   size_t no_classes;
   size_t no_reads=splicing_vector_int_size(position);
@@ -384,6 +413,12 @@ int splicing_solve_gene_paired(const splicing_gff_t *gff, size_t gene,
   splicing_matrix_t *mymatch_matrix=match_matrix, vmatch_matrix;
   splicing_matrix_t *myass_matrix=assignment_matrix, vass_matrix;
 
+  if (!nomatch) {
+    mynomatch=&vnomatch;
+    SPLICING_CHECK(splicing_vector_init(mynomatch, 0));
+    SPLICING_FINALLY(splicing_vector_destroy, mynomatch);
+  }
+
   if (!match_matrix) {
     mymatch_matrix=&vmatch_matrix;
     SPLICING_CHECK(splicing_matrix_init(mymatch_matrix, 0, 0));
@@ -437,6 +472,20 @@ int splicing_solve_gene_paired(const splicing_gff_t *gff, size_t gene,
   SPLICING_CHECK(splicing_matrix_transpose(&A));
   SPLICING_CHECK(splicing_vector_long_init(&index, 0));
   SPLICING_FINALLY(splicing_vector_long_destroy, &index);
+  splicing_vector_update(mynomatch, &match);
+  
+  SPLICING_CHECK(splicing_nnls(&A, &match, expression, &rnorm, &index,
+			       &nsetp));
+
+  if (residuals) {
+    int i, n=splicing_vector_size(&match);
+    splicing_dgemv(/*transpose=*/ 1, /*alpha=*/ 1.0, myass_matrix, 
+		   expression, /*beta=*/ 0.0, &match);
+    SPLICING_CHECK(splicing_vector_resize(residuals, no_classes));
+    for (i=0; i<n; i++) {
+      VECTOR(*residuals)[i] = VECTOR(*mynomatch)[i] - VECTOR(match)[i];
+    }
+  }
 
   if (!assignment_matrix) {
     splicing_matrix_destroy(myass_matrix);
@@ -445,17 +494,21 @@ int splicing_solve_gene_paired(const splicing_gff_t *gff, size_t gene,
   if (!match_matrix) {
     splicing_matrix_destroy(mymatch_matrix);
     SPLICING_FINALLY_CLEAN(1);
+  }  
+  if (!nomatch) { 
+    splicing_vector_destroy(mynomatch);
+    SPLICING_FINALLY_CLEAN(1);
   }
-  
-  SPLICING_CHECK(splicing_nnls(&A, &match, expression, &rnorm, &index,
-			       &nsetp));
-  
+
   splicing_vector_long_destroy(&index);
   splicing_matrix_destroy(&A);
   splicing_vector_destroy(&match);
   SPLICING_FINALLY_CLEAN(3);
 
-  splicing_vector_scale(expression, 1.0/splicing_vector_sum(expression));
+  if (scale) {
+    double fac=1.0/splicing_vector_sum(expression);
+    splicing_vector_scale(expression, fac);
+  }
 
   return 0;
   
